@@ -18,6 +18,7 @@ from __future__ import division
 from __future__ import print_function
 
 import abc
+import collections
 import csv
 import io
 import os
@@ -37,6 +38,92 @@ except ImportError:
   ENABLE_METADATA = False
 
 
+class MetadataWriter:
+  """Helper class to populate Audio Metadata, to be used in `with` statement.
+
+  Simple usage for model with two classification heads.
+
+  with MetadataWriter(tflite_path) as writer:
+    writer.add_input(sample_rate=16000, channels=1)
+    writer.add_output(name='animal_sound', labels=['dog', 'cat'])
+    writer.add_output(name='speech_command', labels=['yes', 'no'])
+    writer.save(tflite_path, json_filepath)
+
+  `add_output` can also take an ordered dict for multiple locales, example:
+
+  writer.add_output(name='animal_sound', labels=collections.OrderedDict([
+    ('en', ['bird', 'cat']),
+    ('fr', ['oiseau', 'chat'])
+  ]))
+  """
+
+  def __init__(self, tflite_filepath, **kwargs):
+    self._model = writer_utils.load_file(tflite_filepath)
+    self._general_md = md_info.GeneralMd(**kwargs)
+    self._inputs = []
+    self._outputs = []
+
+  def __enter__(self):
+    self._temp_folder = tempfile.TemporaryDirectory()
+    return self
+
+  def __exit__(self, exc_type, exc_val, exc_tb):
+    self._temp_folder.cleanup()
+    # Delete the attribute so that it errors out if not in `with` statement.
+    delattr(self, '_temp_folder')
+
+  def add_input(self, **kwargs):
+    """Add metadta for the input tensor."""
+    self._inputs.append(md_info.InputAudioTensorMd(**kwargs))
+
+  def add_output(self, name, labels, **kwargs):
+    """Add metadata for output tensor in order."""
+    if isinstance(labels, list):
+      default_locale = None
+      labels = collections.OrderedDict([(default_locale, labels)])
+      return self.add_output(name, labels, **kwargs)
+
+    label_files = []
+    if isinstance(labels, collections.OrderedDict):
+      for locale, label_list in labels.items():
+        full_path = os.path.join(
+            self._temp_folder.name,
+            '{}_labels_{}.txt'.format(name, locale or 'default'))
+        model_util.export_labels(full_path, label_list)
+        label_files.append(
+            md_info.LabelFileMd(file_path=full_path, locale=locale))
+    else:
+      raise ValueError(
+          '`labels` should be either a list of labels or an ordered dict mapping `locale` -> list of labels. got: {}'
+          .format(labels))
+
+    idx = len(self._outputs)
+    self._outputs.append(
+        md_info.ClassificationTensorMd(
+            name=name,
+            label_files=label_files,
+            tensor_type=writer_utils.get_output_tensor_types(self._model)[idx],
+            **kwargs))
+
+  def save(self, tflite_filepath=None, json_filepath=None):
+    """Persist model with metadata."""
+    if len(self._inputs) > 1:
+      raise ValueError('Only supports single input, got {}'.format(
+          len(self._inputs)))
+    input_md = self._inputs[0]
+
+    writer = md_writer.MetadataWriter.create_from_metadata_info_for_multihead(
+        model_buffer=self._model,
+        general_md=self._general_md,
+        input_md=input_md,
+        output_md_list=self._outputs)
+    if tflite_filepath:
+      writer_utils.save_file(writer.populate(), tflite_filepath, mode='wb')
+    if json_filepath:
+      writer_utils.save_file(
+          writer.get_metadata_json(), json_filepath, mode='wt')
+
+
 def _ensure_tf25(version):
   if version < '2.5':
     raise RuntimeError(
@@ -52,8 +139,6 @@ def _get_tf_version():
 class BaseSpec(abc.ABC):
   """Base model spec for audio classification."""
 
-  compat_tf_versions = (2,)
-
   def __init__(self, model_dir=None, strategy=None):
     _ensure_tf25(_get_tf_version())
     self.model_dir = model_dir
@@ -131,37 +216,74 @@ def _load_tfjs_speech_command_model():
 class BrowserFFTSpec(BaseSpec):
   """Model good at detecting speech commands, using Browser FFT spectrum."""
 
+  EXPECTED_WAVEFORM_LENGTH = 44032
+
+  # Information used to populate TFLite metadata.
+  _MODEL_NAME = 'AudioClassifier'
+  _MODEL_DESCRIPTION = ('Identify the most prominent type in the audio clip '
+                        'from a known set of categories.')
+
+  _MODEL_VERSION = 'v1'
+  _MODEL_AUTHOR = 'TensorFlow Lite Model Maker'
+  _MODEL_LICENSES = ('Apache License. Version 2.0 '
+                     'http://www.apache.org/licenses/LICENSE-2.0.')
+
+  _SAMPLE_RATE = 44100
+  _CHANNELS = 1
+
+  _INPUT_NAME = 'audio_clip'
+  _INPUT_DESCRIPTION = 'Input audio clip to be classified.'
+
+  _OUTPUT_NAME = 'probability'
+  _OUTPUT_DESCRIPTION = 'Scores of the labels respectively.'
+
   def __init__(self, model_dir=None, strategy=None):
+    """Initialize a new instance for BrowserFFT spec.
+
+    Args:
+      model_dir: The location to save the model checkpoint files.
+      strategy: An instance of TF distribute strategy. If none, it will use the
+        default strategy (either SingleDeviceStrategy or the current scoped
+        strategy.
+    """
     super(BrowserFFTSpec, self).__init__(model_dir, strategy)
     self._preprocess_model = _load_browser_fft_preprocess_model()
     self._tfjs_sc_model = _load_tfjs_speech_command_model()
 
-    self.expected_waveform_len = self._preprocess_model.input_shape[-1]
-
   @property
   def target_sample_rate(self):
     return 44100
 
-  @tf.function
+  @tf.function(input_signature=[
+      tf.TensorSpec(shape=[None], dtype=tf.float32),
+      tf.TensorSpec([], dtype=tf.int32)
+  ])
   def _ensure_length(self, wav, unused_label):
-    return len(wav) >= self.expected_waveform_len
+    return len(wav) >= self.EXPECTED_WAVEFORM_LENGTH
 
-  @tf.function
+  @tf.function(input_signature=[
+      tf.TensorSpec(shape=[None], dtype=tf.float32),
+      tf.TensorSpec([], dtype=tf.int32)
+  ])
   def _split(self, wav, label):
     """Split the long audio samples into multiple trunks."""
     # wav shape: (audio_samples, )
-    chunks = tf.math.floordiv(len(wav), self.expected_waveform_len)
-    unused = tf.math.floormod(len(wav), self.expected_waveform_len)
+    chunks = tf.math.floordiv(len(wav), self.EXPECTED_WAVEFORM_LENGTH)
+    unused = tf.math.floormod(len(wav), self.EXPECTED_WAVEFORM_LENGTH)
     # Drop unused data
     wav = wav[:len(wav) - unused]
     # Split the audio sample into multiple chunks
-    wav = tf.reshape(wav, (chunks, 1, self.expected_waveform_len))
+    wav = tf.reshape(wav, (chunks, 1, self.EXPECTED_WAVEFORM_LENGTH))
 
     return wav, tf.repeat(tf.expand_dims(label, 0), len(wav))
 
-  @tf.function
+  @tf.function(input_signature=[
+      tf.TensorSpec(shape=[1, EXPECTED_WAVEFORM_LENGTH], dtype=tf.float32),
+      tf.TensorSpec([], dtype=tf.int32)
+  ])
   def _preprocess(self, x, label):
-    # x has shape (1, expected_waveform_len)
+    """Preprocess the dataset to extract the spectrum."""
+    # x has shape (1, EXPECTED_WAVEFORM_LENGTH)
     spectrum = self._preprocess_model(x)
     # y has shape (1, embedding_len)
     spectrum = tf.squeeze(spectrum, axis=0)
@@ -205,6 +327,40 @@ class BrowserFFTSpec(BaseSpec):
         train_ds, validation_data=validation_ds, epochs=epochs, **kwargs)
     return hist
 
+  def create_serving_model(self, training_model):
+    """Create a model for serving."""
+    combined = tf.keras.Sequential()
+    combined.add(self._preprocess_model)
+    combined.add(training_model)
+    # Build the model.
+    combined.build([None, self.EXPECTED_WAVEFORM_LENGTH])
+    return combined
+
+  def _export_metadata(self, tflite_filepath, index_to_label,
+                       export_metadata_json_file):
+    """Export TFLite metadata."""
+    with MetadataWriter(
+        tflite_filepath,
+        name=self._MODEL_NAME,
+        description=self._MODEL_DESCRIPTION,
+        version=self._MODEL_VERSION,
+        author=self._MODEL_AUTHOR,
+        licenses=self._MODEL_LICENSES) as writer:
+      writer.add_input(
+          name=self._INPUT_NAME,
+          description=self._INPUT_DESCRIPTION,
+          sample_rate=self._SAMPLE_RATE,
+          channels=self._CHANNELS)
+
+      writer.add_output(
+          labels=index_to_label,
+          name=self._OUTPUT_NAME,
+          description=self._OUTPUT_DESCRIPTION)
+
+      json_filepath = (os.path.splitext(tflite_filepath)[0] +
+                       '.json') if export_metadata_json_file else None
+      writer.save(tflite_filepath, json_filepath)
+
   def export_tflite(self,
                     model,
                     tflite_filepath,
@@ -226,26 +382,25 @@ class BrowserFFTSpec(BaseSpec):
         only if `with_metadata` is True.
       index_to_label: A list that map from index to label class name.
     """
-    del with_metadata, export_metadata_json_file, index_to_label
-    combined = tf.keras.Sequential()
-    combined.add(self._preprocess_model)
-    combined.add(model)
-    # Build the model.
-    combined.build([None, self.expected_waveform_len])
+    combined = self.create_serving_model(model)
 
     # Sets batch size from None to 1 when converting to tflite.
     model_util.set_batch_size(model, batch_size=1)
 
     model_util.export_tflite(
-        combined,
-        tflite_filepath,
-        quantization_config=None,
-        supported_ops=(tf.lite.OpsSet.TFLITE_BUILTINS,
-                       tf.lite.OpsSet.SELECT_TF_OPS))
+        combined, tflite_filepath, quantization_config=None)
 
     # Sets batch size back to None to support retraining later.
     model_util.set_batch_size(model, batch_size=None)
 
+    if with_metadata:
+      if not ENABLE_METADATA:
+        print('Writing Metadata is not support in the installed tflite-support '
+              'version. Please use tflite-support >= 0.2.*')
+      else:
+        self._export_metadata(tflite_filepath, index_to_label,
+                              export_metadata_json_file)
+
 
 @mm_export('audio_classifier.YamNetSpec')
 class YAMNetSpec(BaseSpec):
@@ -268,15 +423,13 @@ class YAMNetSpec(BaseSpec):
   _INPUT_NAME = 'audio_clip'
   _INPUT_DESCRIPTION = 'Input audio clip to be classified.'
 
-  _YAMNET_OUTPUT_NAME = 'scores'
+  _YAMNET_OUTPUT_NAME = 'yamnet'
   _YAMNET_OUTPUT_DESCRIPTION = ('Scores in range 0..1.0 for each of the 521 '
                                 'output classes.')
-  _YAMNET_LABEL_FILE = 'yamnet_label_list.txt'
 
-  _CUSTOM_OUTPUT_NAME = 'classification'
+  _CUSTOM_OUTPUT_NAME = 'custom'
   _CUSTOM_OUTPUT_DESCRIPTION = (
       'Scores in range 0..1.0 for each output classes.')
-  _CUSTOM_LABEL_FILE = 'custom_label_list.txt'
 
   def __init__(
       self,
@@ -286,6 +439,22 @@ class YAMNetSpec(BaseSpec):
       frame_length=EXPECTED_WAVEFORM_LENGTH,  # Window size 0.975 s
       frame_step=EXPECTED_WAVEFORM_LENGTH // 2,  # Hop of 0.975 /2 s
       keep_yamnet_and_custom_heads=True):
+    """Initialize a new instance for YAMNet spec.
+
+    Args:
+      model_dir: The location to save the model checkpoint files.
+      strategy: An instance of TF distribute strategy. If none, it will use the
+        default strategy (either SingleDeviceStrategy or the current scoped
+        strategy.
+      yamnet_model_handle: Path of the TFHub model for retrining.
+      frame_length: The number of samples in each audio frame. If the audio file
+        is shorter than `frame_length`, then the audio file will be ignored.
+      frame_step: The number of samples between two audio frames. This value
+        should be bigger than `frame_length`.
+      keep_yamnet_and_custom_heads: Boolean, decides if the final TFLite model
+        contains both YAMNet and custom trained classification heads. When set
+        to False, only the trained custom head will be preserved.
+    """
     super(YAMNetSpec, self).__init__(model_dir, strategy)
     self._yamnet_model_handle = yamnet_model_handle
     self._yamnet_model = hub.load(yamnet_model_handle)
@@ -299,8 +468,8 @@ class YAMNetSpec(BaseSpec):
 
   def create_model(self, num_classes, train_whole_model=False):
     model = tf.keras.Sequential([
-        tf.keras.layers.Input(
-            shape=(YAMNetSpec.EMBEDDING_SIZE),
+        tf.keras.layers.InputLayer(
+            input_shape=(YAMNetSpec.EMBEDDING_SIZE),
             dtype=tf.float32,
             name='embedding'),
         tf.keras.layers.Dense(
@@ -373,76 +542,57 @@ class YAMNetSpec(BaseSpec):
     class_names = class_names[1:]  # Skip CSV header
     return class_names
 
-  def _export_labels(self, filepath, index_to_label):
-    with tf.io.gfile.GFile(filepath, 'w') as f:
-      f.write('\n'.join(index_to_label))
-
   def _export_metadata(self, tflite_filepath, index_to_label,
                        export_metadata_json_file):
     """Export TFLite metadata."""
-    with tempfile.TemporaryDirectory() as temp_dir:
-      # Prepare metadata
-      with open(tflite_filepath, 'rb') as f:
-        model_buffer = f.read()
-
-      general_md = md_info.GeneralMd(
-          name=self._MODEL_NAME,
-          description=self._MODEL_DESCRIPTION,
-          version=self._MODEL_VERSION,
-          author=self._MODEL_AUTHOR,
-          licenses=self._MODEL_LICENSES)
-      input_md = md_info.InputAudioTensorMd(self._INPUT_NAME,
-                                            self._INPUT_DESCRIPTION,
-                                            self._SAMPLE_RATE, self._CHANNELS)
-
-      # Save label files.
-      custom_label_filepath = os.path.join(temp_dir, self._CUSTOM_LABEL_FILE)
-      self._export_labels(custom_label_filepath, index_to_label)
-
-      custom_output_md = md_info.ClassificationTensorMd(
-          name=self._CUSTOM_OUTPUT_NAME,
-          description=self._CUSTOM_OUTPUT_DESCRIPTION,
-          label_files=[
-              md_info.LabelFileMd(
-                  file_path=os.path.join(temp_dir, self._CUSTOM_LABEL_FILE))
-          ],
-          tensor_type=writer_utils.get_output_tensor_types(model_buffer)[-1],
-          score_calibration_md=None)
+    with MetadataWriter(
+        tflite_filepath,
+        name=self._MODEL_NAME,
+        description=self._MODEL_DESCRIPTION,
+        version=self._MODEL_VERSION,
+        author=self._MODEL_AUTHOR,
+        licenses=self._MODEL_LICENSES) as writer:
+      writer.add_input(
+          name=self._INPUT_NAME,
+          description=self._INPUT_DESCRIPTION,
+          sample_rate=self._SAMPLE_RATE,
+          channels=self._CHANNELS)
 
       if self._keep_yamnet_and_custom_heads:
-        yamnet_label_filepath = os.path.join(temp_dir, self._YAMNET_LABEL_FILE)
-        self._export_labels(yamnet_label_filepath, self._yamnet_labels())
-
-        yamnet_output_md = md_info.ClassificationTensorMd(
+        writer.add_output(
+            labels=self._yamnet_labels(),
             name=self._YAMNET_OUTPUT_NAME,
-            description=self._YAMNET_OUTPUT_DESCRIPTION,
-            label_files=[
-                md_info.LabelFileMd(
-                    file_path=os.path.join(temp_dir, self._YAMNET_LABEL_FILE))
-            ],
-            tensor_type=writer_utils.get_output_tensor_types(model_buffer)[0],
-            score_calibration_md=None)
-        output_md = [yamnet_output_md, custom_output_md]
-      else:
-        output_md = [custom_output_md]
+            description=self._YAMNET_OUTPUT_DESCRIPTION)
 
-      # Populate metadata
-      writer = md_writer.MetadataWriter.create_from_metadata_info_for_multihead(
-          model_buffer=model_buffer,
-          general_md=general_md,
-          input_md=input_md,
-          output_md_list=output_md)
+      writer.add_output(
+          labels=index_to_label,
+          name=self._CUSTOM_OUTPUT_NAME,
+          description=self._CUSTOM_OUTPUT_DESCRIPTION)
 
-      output_model = writer.populate()
+      json_filepath = (os.path.splitext(tflite_filepath)[0] +
+                       '.json') if export_metadata_json_file else None
+      writer.save(tflite_filepath, json_filepath)
 
-      with tf.io.gfile.GFile(tflite_filepath, 'wb') as f:
-        f.write(output_model)
+  def create_serving_model(self, training_model):
+    """Create a model for serving."""
+    embedding_extraction_layer = hub.KerasLayer(
+        self._yamnet_model_handle, trainable=False)
+    keras_input = tf.keras.Input(
+        shape=(YAMNetSpec.EXPECTED_WAVEFORM_LENGTH,),
+        dtype=tf.float32,
+        name='audio')  # (1, wav)
+    reshaped_input = tf.reshape(keras_input,
+                                (YAMNetSpec.EXPECTED_WAVEFORM_LENGTH,))  # (wav)
 
-      if export_metadata_json_file:
-        metadata_json = writer.get_metadata_json()
-        export_json_file = os.path.splitext(tflite_filepath)[0] + '.json'
-        with open(export_json_file, 'w') as f:
-          f.write(metadata_json)
+    scores, embeddings, _ = embedding_extraction_layer(reshaped_input)
+    serving_outputs = training_model(embeddings)
+
+    if self._keep_yamnet_and_custom_heads:
+      serving_model = tf.keras.Model(keras_input, [scores, serving_outputs])
+    else:
+      serving_model = tf.keras.Model(keras_input, serving_outputs)
+
+    return serving_model
 
   def export_tflite(self,
                     model,
@@ -466,32 +616,12 @@ class YAMNetSpec(BaseSpec):
         only if `with_metadata` is True.
       index_to_label: A list that map from index to label class name.
     """
-    embedding_extraction_layer = hub.KerasLayer(
-        self._yamnet_model_handle, trainable=False)
-
-    keras_input = tf.keras.Input(
-        shape=(YAMNetSpec.EXPECTED_WAVEFORM_LENGTH,),
-        dtype=tf.float32,
-        name='audio')  # (1, wav)
-    reshaped_input = tf.reshape(keras_input,
-                                (YAMNetSpec.EXPECTED_WAVEFORM_LENGTH,))  # (wav)
-
-    scores, embeddings, _ = embedding_extraction_layer(reshaped_input)
-    serving_outputs = model(embeddings)
-
-    if self._keep_yamnet_and_custom_heads:
-      serving_model = tf.keras.Model(keras_input, [scores, serving_outputs])
-    else:
-      serving_model = tf.keras.Model(keras_input, serving_outputs)
+    serving_model = self.create_serving_model(model)
 
     # TODO(b/164229433): Remove SELECT_TF_OPS once changes in the bug are
     # released.
     model_util.export_tflite(
-        serving_model,
-        tflite_filepath,
-        quantization_config=None,
-        supported_ops=(tf.lite.OpsSet.TFLITE_BUILTINS,
-                       tf.lite.OpsSet.SELECT_TF_OPS))
+        serving_model, tflite_filepath, quantization_config=None)
 
     if with_metadata:
       if not ENABLE_METADATA:
