@@ -40,12 +40,13 @@ from __future__ import division
 from __future__ import print_function
 
 import os
+
 from absl import app
 from absl import flags
 from absl import logging
-
 import tensorflow as tf
 from tflite_model_maker import audio_classifier
+from tflite_model_maker import ExportFormat
 from tflite_model_maker import model_spec
 
 FLAGS = flags.FLAGS
@@ -116,11 +117,11 @@ def run(spec,
     validation_data = audio_classifier.DataLoader.from_esc50(
         spec, data_dir, folds=[
             4,
-        ], categories=categories)
+        ], categories=categories, cache=True)
     test_data = audio_classifier.DataLoader.from_esc50(
         spec, data_dir, folds=[
             5,
-        ], categories=categories)
+        ], categories=categories, cache=True)
   elif dataset_type == 'bird':
     if isinstance(spec, audio_classifier.YamNetSpec):
       # In some files, two consecutive bird sounds might be too far apart, so
@@ -141,7 +142,7 @@ def run(spec,
         spec, os.path.join(data_dir, 'test'), cache=True)
 
   else:
-    data = audio_classifier.DataLoader.from_folder(spec, data_dir)
+    data = audio_classifier.DataLoader.from_folder(spec, data_dir, cache=True)
     train_data, rest_data = data.split(0.8)
     validation_data, test_data = rest_data.split(0.5)
 
@@ -161,13 +162,16 @@ def run(spec,
   print(model.confusion_matrix(test_data))
   print('labels: ', test_data.index_to_label)
 
-  print('\nExporing the TFLite model to {}'.format(export_dir))
-  model.export(export_dir)
+  print('\nExporing the Saved model and TFLite model to {}'.format(export_dir))
+  model.export(
+      export_dir, export_format=(ExportFormat.TFLITE, ExportFormat.SAVED_MODEL))
 
 
 def main(_):
   logging.set_verbosity(logging.INFO)
 
+  export_dir = os.path.expanduser(FLAGS.export_dir)
+
   if not FLAGS.data_dir:
     if FLAGS.dataset == 'esc50':
       data_dir = download_esc50_dataset()
@@ -179,9 +183,9 @@ def main(_):
     else:
       raise ValueError('Unsupported dataset type: ', FLAGS.dataset)
   else:
-    data_dir = FLAGS.data_dir
+    data_dir = os.path.expanduser(FLAGS.data_dir)
 
-  run(FLAGS.spec, data_dir, FLAGS.dataset, export_dir=FLAGS.export_dir)
+  run(FLAGS.spec, data_dir, FLAGS.dataset, export_dir=export_dir)
 
 
 if __name__ == '__main__':
